-- ============================================================================
-- == HTBrokerSupport - Provides support for integrating this addon into     ==
-- ==                   LibDataBroker (aka: LDB) supporting display addons.  ==
-- ============================================================================

HTBrokerSupport    = LibStub("AceAddon-3.0"):NewAddon("HTBrokerSupport", "AceConsole-3.0", "AceEvent-3.0")
local L            = LibStub("AceLocale-3.0"):GetLocale("HalonaTools")

local DropDownMenu = CreateFrame("Frame", "DropDownMenu")       -- Dropdown Menu
local info         = {}                                 -- Dropdown Menu Options

-- ============================================================================
-- == OnInitialize()                                                         ==
-- ==                                                                        ==
-- == This function is called when the addon is initialised.                 ==
-- ============================================================================

function HTBrokerSupport:OnInitialize()
end

-- ============================================================================
-- == OnEnable()                                                             ==
-- ==                                                                        ==
-- == This function is called when the addon is enabled.                     ==
-- ============================================================================

function HTBrokerSupport:OnEnable()

  HTBrokerSupport:CreateDataObject()

end

-- ============================================================================
-- == OnEnable()                                                             ==
-- ==                                                                        ==
-- == This function is called when the addon is disabled.                    ==
-- ============================================================================

function HTBrokerSupport:OnDisable()
end

-- ============================================================================
-- == CreateDataObject()                                                     ==
-- ==                                                                        ==
-- == This function creates the LDB data object.                             ==
-- ============================================================================

function HTBrokerSupport:CreateDataObject()

  LibStub:GetLibrary("LibDataBroker-1.1"):NewDataObject("HalonaTools", {
    type = "launcher",
    label = L["addon-name"],
    OnClick = function(_, msg)
      if msg == "LeftButton" then
        HalonaTools:JoinGuildRaidEvent()
      elseif msg == "RightButton" then
        HTBrokerSupport:ShowMenu()
      end
    end,
    icon = "Interface\\AddOns\\HalonaTools\\Images\\icon",
    OnTooltipShow = function(tooltip)
      if not tooltip or not tooltip.AddLine then return end
      tooltip:AddLine("|r" .. L["addon-name"] .. " (" .. L["addon-version-number"] .. ")|r")
      tooltip:AddLine(" ")
      tooltip:AddLine(L["addon-desc-01"])
      tooltip:AddLine(L["addon-desc-02"])
      tooltip:AddLine(" ")
      tooltip:AddLine("|cff00c0ff" .. L["ldb-tooltip-heading-guildinfo"] .. "|r")
      tooltip:AddDoubleLine("|cffffffff" .. L["ldb-tooltip-item-name-name"]  .. "|r", "|cff00ff00" .. HTCommon:GetGuildName()     .."|r")
      tooltip:AddDoubleLine("|cffffffff" .. L["ldb-tooltip-item-name-rank"]  .. "|r", "|cff00ff00" .. HTCommon:GetGuildRankName() .."|r")
      tooltip:AddDoubleLine("|cffffffff" .. L["ldb-tooltip-item-name-level"] .. "|r", "|cff00ff00" .. HTCommon:GetGuildLevel()    .."|r")
      tooltip:AddLine(" ")
      tooltip:AddLine("|cff00c0ff" .. L["ldb-tooltip-heading-guildofficers"] .. "|r")

      local guildOfficerList = HTBrokerSupport:GetTooltipOfficerList()

      for i,v in ipairs(guildOfficerList) do
        tooltip:AddLine(v)
      end

      tooltip:AddLine(" ")
      tooltip:AddLine("|cff00c0ff" .. L["ldb-tooltip-heading-tools"] .. "|r")
      tooltip:AddDoubleLine("|cffffffff" .. L["ldb-tooltip-item-name-autoinvitations"]  .. "|r", HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledAutoInvitations(nil)))
      tooltip:AddDoubleLine("|cffffffff" .. L["ldb-tooltip-item-name-buffannouncer"]    .. "|r", HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledBuffAnnouncer(nil)))
      tooltip:AddDoubleLine("|cffffffff" .. L["ldb-tooltip-item-name-buffmonitor"]      .. "|r", HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledBuffMonitor(nil)))
      tooltip:AddDoubleLine("|cffffffff" .. L["ldb-tooltip-item-name-busyresponder"]    .. "|r", "|cffffff00" .. " (".. HTConsole:BusyResponderGetNumberOfMessagesStored() .. L["ldb-tooltip-item-text-busyresponder"] .. ") " .. "|r" .. HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledBusyResponder(nil)) )

      tooltip:AddDoubleLine("|cffffffff" .. L["ldb-tooltip-item-name-rms-timer"]  .. "|r", "|cffffff00" .. " (".. HTData:SecondsToMinutes(HTConsole:GetRMSTimer()) .. ") " .. "|r" .. HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledRecruitmentMessageSender(nil)))
      tooltip:AddDoubleLine("|cffffffff" .. "- " .. L["ldb-tooltip-item-name-rms-grouplock"]  .. "|r", HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledRMSGroupLock(nil)))

      tooltip:AddDoubleLine("|cffffffff" .. "- " .. L["ldb-tooltip-item-name-rms-prefguild"]  .. "|r", "|cffffff00" .. HTConfig:GetPreferredRMSGuild(nil) .. "|r")
      tooltip:AddDoubleLine("|cffffffff" .. "- " .. L["ldb-tooltip-item-name-rms-channel"]  .. "|r", "|cffffff00" .. HTConsole:GetRMSChannelNumber() .. " (" .. HTConsole:GetRMSChannelNameByNumber(HTConsole:GetRMSChannelNumber()) .. ")|r")

      tooltip:AddDoubleLine("|cffffffff" .. L["ldb-tooltip-item-name-heartbeat-timer"]  .. "|r", "|cffffff00" .. " (".. HTData:SecondsToMinutes(HTConsole:GetHeartbeatTimer()) .. " / " ..  HTData:SecondsToMinutes(HTConfig:GetHeartbeatInterval(nil)) .. ")|r")

      if HTConfig:GetIsEnabledDebugMode() then
        tooltip:AddDoubleLine("|cffffffff" .. L["ldb-tooltip-item-name-debugmode-timer"]    .. "|r", HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledDebugMode(nil)))
        tooltip:AddDoubleLine("|cffffffff" .. L["ldb-tooltip-item-name-debugfilter-timer"]  .. "|r", "|cffffff00" .. " (".. HTConfig:GetDebugFilter(nil) .. ") " .. "|r" .. HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledDebugFilter(nil)))
      end

      tooltip:AddLine(" ")
      tooltip:AddLine("|cff00c0ff" .. L["ldb-tooltip-heading-actions"] .. "|r")
      tooltip:AddDoubleLine("|cffeda55f" .. L["ldb-tooltip-item-name-leftclick"]  .. "|r", "|cff00ff00" .. L["ldb-tooltip-item-value-leftclick"] .. "|r")
      tooltip:AddDoubleLine("|cffeda55f" .. L["ldb-tooltip-item-name-rightclick"] .. "|r", "|cff00ff00" .. L["ldb-tooltip-item-value-rightclick"]  .. "|r")
    end,
  })

end

-- ============================================================================
-- == ShowMenu()                                                             ==
-- ==                                                                        ==
-- == This function displays the drop-down menu.                             ==
-- ============================================================================

function HTBrokerSupport:ShowMenu()

  HTBrokerSupport:BuildDropDownMenu()

  ToggleDropDownMenu(1, nil, DropDownMenu, "cursor", 0, 0)

end

-- ============================================================================
-- == BuildDropDownMenu()                                                    ==
-- ==                                                                        ==
-- == This function builds the drop-down menu. It is a dynamic function and  ==
-- == can be called either once only for menus that are not expected to      ==
-- == change during a session or every time before the menu is displayed for ==
-- == menus that change often.                                               ==
-- ============================================================================

function HTBrokerSupport:BuildDropDownMenu()

  DropDownMenu.displayMode = "MENU"
  DropDownMenu.initialize  = function(self, level)

  if not level then return end
  wipe(info)

  -- ==========================================================================
  -- == Level 1 - Top level menu items.                                      ==
  -- ==========================================================================
  if level == 1 then

    -- General Menu Defaults.
    HTBrokerSupport:AddMenuDefaultConfig()

    -- Create the title of the menu
    HTBrokerSupport:AddMenuTitle(L["ldb-menu-main-name"])
    UIDropDownMenu_AddButton(info, level)

    -- Empty Menu Item.
    HTBrokerSupport:AddMenuDummyButton()
    UIDropDownMenu_AddButton(info, level)

    -- Buff-Announcer Submenu.
    HTBrokerSupport:AddMenuSubMenu("SubmenuBuffAnnouncer")
    HTBrokerSupport:AddMenuButtonText("ldb-menu-heading-buffannouncer")
    UIDropDownMenu_AddButton(info, level)

    -- Buff-Monitor Submenu.
    HTBrokerSupport:AddMenuSubMenu("SubmenuBuffMonitor")
    HTBrokerSupport:AddMenuButtonText("ldb-menu-heading-buffmonitor")
    UIDropDownMenu_AddButton(info, level)

    -- Busy Responder Submenu.
    HTBrokerSupport:AddMenuSubMenu("SubmenuBusyResponder")
    HTBrokerSupport:AddMenuButtonText("ldb-menu-heading-busyresponder")
    UIDropDownMenu_AddButton(info, level)

    -- Invitations Submenu.
    HTBrokerSupport:AddMenuSubMenu("SubmenuInvitations")
    HTBrokerSupport:AddMenuButtonText("ldb-menu-heading-invitations")
    UIDropDownMenu_AddButton(info, level)

    -- Raid Functions Submenu.
    HTBrokerSupport:AddMenuSubMenu("SubmenuRaidFunctions")
    HTBrokerSupport:AddMenuButtonText("ldb-menu-heading-raid")
    UIDropDownMenu_AddButton(info, level)

    -- Recruitment Submenu.
    HTBrokerSupport:AddMenuSubMenu("SubmenuRecruitment")
    HTBrokerSupport:AddMenuButtonText("ldb-menu-heading-recruitment")
    UIDropDownMenu_AddButton(info, level)

    -- System Submenu.
    HTBrokerSupport:AddMenuSubMenu("SubmenuSystem")
    HTBrokerSupport:AddMenuButtonText("ldb-menu-heading-system")
    UIDropDownMenu_AddButton(info, level)

    -- Empty Menu Item.
    HTBrokerSupport:AddMenuDummyButton()
    UIDropDownMenu_AddButton(info, level)

    -- About Menu Item.
    HTBrokerSupport:AddMenuPushButton()
    HTBrokerSupport:AddMenuButtonText("ldb-menu-item-about")
    info.func = function() HTGUI:About() end
    UIDropDownMenu_AddButton(info, level)

    -- Close Menu Item.
    HTBrokerSupport:AddMenuPushButton()
    HTBrokerSupport:AddMenuButtonText("ldb-menu-item-closemenu")
    info.func = function() CloseDropDownMenus() end
    UIDropDownMenu_AddButton(info, level)

  -- ==========================================================================
  -- == Level 2 - Submenus directly linked from the main menu.               ==
  -- ==========================================================================
  elseif level == 2 then

    -- ========================================================================
    -- == Render the level 2 submenu for: Buff-Announcer ======================
    -- ========================================================================
    if UIDROPDOWNMENU_MENU_VALUE == "SubmenuBuffAnnouncer" then

      -- General Menu Defaults.
      HTBrokerSupport:AddMenuDefaultConfig()

      -- Create the title of the menu
      HTBrokerSupport:AddMenuTitle(L["ldb-menu-buffannouncer-name"])
      UIDropDownMenu_AddButton(info, level)

      -- Empty Menu Item.
      HTBrokerSupport:AddMenuDummyButton()
      UIDropDownMenu_AddButton(info, level)

      -- Create the title of the menu
      HTBrokerSupport:AddMenuTitle(L["ldb-menu-quick-config-name"])
      UIDropDownMenu_AddButton(info, level)

      -- Do BuffAnnouncer Enabled Toggle.
      HTBrokerSupport:AddMenuToggleButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-ba-enabled")
      info.checked = HTBrokerSupport:IsEnabled(HTConfig:GetIsEnabledBuffAnnouncer(nil))
      info.func    = function()
                       HTConfig:ToggleIsEnabledBuffAnnouncer()
                       HTBrokerSupport:DisplayStatus(L["ldb-menu-item-ba-enabled-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledBuffAnnouncer(nil)))
                     end
      UIDropDownMenu_AddButton(info, level)

      -- Do BuffAnnouncer Enabled Toggle.
      HTBrokerSupport:AddMenuToggleButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-ba-food-enabled")
      info.checked = HTBrokerSupport:IsEnabled(HTConfig:GetIsEnabledBuffAnnouncerFood(nil))
      info.func    = function()
                       HTConfig:ToggleIsEnabledBuffAnnouncerFood()
                       HTBrokerSupport:DisplayStatus(L["ldb-menu-item-ba-food-enabled-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledBuffAnnouncerFood(nil)))
                     end
      UIDropDownMenu_AddButton(info, level)

      -- Do BuffAnnouncer Enabled Toggle.
      HTBrokerSupport:AddMenuToggleButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-ba-cauldron-enabled")
      info.checked = HTBrokerSupport:IsEnabled(HTConfig:GetIsEnabledBuffAnnouncerCauldron(nil))
      info.func    = function()
                       HTConfig:ToggleIsEnabledBuffAnnouncerCauldron()
                       HTBrokerSupport:DisplayStatus(L["ldb-menu-item-ba-cauldron-enabled-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledBuffAnnouncerCauldron(nil)))
                     end
      UIDropDownMenu_AddButton(info, level)

      -- Do BuffAnnouncer Enabled Toggle.
      HTBrokerSupport:AddMenuToggleButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-ba-healthstone-enabled")
      info.checked = HTBrokerSupport:IsEnabled(HTConfig:GetIsEnabledBuffAnnouncerHealthStones(nil))
      info.func    = function()
                       HTConfig:ToggleIsEnabledBuffAnnouncerHealthstones()
                       HTBrokerSupport:DisplayStatus(L["ldb-menu-item-ba-healthstone-enabled-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledBuffAnnouncerHealthstones(nil)))
                     end
      UIDropDownMenu_AddButton(info, level)

    end

    -- ========================================================================
    -- == Render the level 2 submenu for: Buff-Monitor ========================
    -- ========================================================================
    if UIDROPDOWNMENU_MENU_VALUE == "SubmenuBuffMonitor" then

      -- General Menu Defaults.
      HTBrokerSupport:AddMenuDefaultConfig()

      -- Create the title of the menu
      HTBrokerSupport:AddMenuTitle(L["ldb-menu-buffmonitor-name"])
      UIDropDownMenu_AddButton(info, level)

      -- Empty Menu Item.
      HTBrokerSupport:AddMenuDummyButton()
      UIDropDownMenu_AddButton(info, level)

      -- Do Self Check.
      HTBrokerSupport:AddMenuPushButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-doselfbuffcheck")
      info.func = function() HTBuffMonitor:DoManualCheck() end
      UIDropDownMenu_AddButton(info, level)

      -- Empty Menu Item.
      HTBrokerSupport:AddMenuDummyButton()
      UIDropDownMenu_AddButton(info, level)

      -- Create the title of the menu
      HTBrokerSupport:AddMenuTitle(L["ldb-menu-quick-config-name"])
      UIDropDownMenu_AddButton(info, level)

      -- Do BuffMonitor Enabled Toggle.
      HTBrokerSupport:AddMenuToggleButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-bm-enabled")
      info.checked = HTBrokerSupport:IsEnabled(HTConfig:GetIsEnabledBuffMonitor(nil))
      info.func    = function()
                       HTConfig:ToggleIsEnabledBuffMonitor()
                       HTBrokerSupport:DisplayStatus(L["ldb-menu-item-bm-enabled-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledBuffMonitor(nil)))
                     end
      UIDropDownMenu_AddButton(info, level)

      -- Do BuffMonitor Check-on-Heartbeat Enabled Toggle.
      HTBrokerSupport:AddMenuToggleButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-bm-on-heartbeat")
      info.checked = HTBrokerSupport:IsEnabled(HTConfig:GetIsEnabledBuffMonitorCheckOnHeartbeat(nil))
      info.func    = function()
                       HTConfig:ToggleIsEnabledBuffMonitorCheckOnHeartbeat()
                       HTBrokerSupport:DisplayStatus(L["ldb-menu-item-bm-on-heartbeat-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledBuffMonitorCheckOnHeartbeat(nil)))
                     end
      UIDropDownMenu_AddButton(info, level)

      -- Do Cataclysm Buffs Acceptable Toggle.
      HTBrokerSupport:AddMenuToggleButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-bm-inc-cata")
      info.checked = HTBrokerSupport:IsEnabled(HTConfig:GetIsEnabledBuffMonitorCataBuffs(nil))
      info.func    = function()
                       HTConfig:ToggleIsEnabledBuffMonitorCataBuffs()
                       HTConfig:UpdateBuffMonitorByExpansion()
                       HTBrokerSupport:DisplayStatus(L["ldb-menu-item-bm-inc-cata-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledBuffMonitorCataBuffs(nil)))
                     end
      UIDropDownMenu_AddButton(info, level)

      -- Do Wrath Buffs Acceptable Toggle.
      HTBrokerSupport:AddMenuToggleButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-bm-inc-wrath")
      info.checked = HTBrokerSupport:IsEnabled(HTConfig:GetIsEnabledBuffMonitorWrathBuffs(nil))
      info.func    = function()
                       HTConfig:ToggleIsEnabledBuffMonitorWrathBuffs()
                       HTConfig:UpdateBuffMonitorByExpansion()
                       HTBrokerSupport:DisplayStatus(L["ldb-menu-item-bm-inc-wrath-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledBuffMonitorWrathBuffs(nil)))
                     end
      UIDropDownMenu_AddButton(info, level)

      -- Do Flask Check Toggle.
      HTBrokerSupport:AddMenuToggleButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-doflaskcheck")
      info.checked = HTBrokerSupport:IsEnabled(HTConfig:GetIsEnabledBuffMonitorCheckFlasks(nil))
      info.func    = function()
                       HTConfig:ToggleIsEnabledBuffMonitorCheckFlasks()
                       HTBrokerSupport:DisplayStatus(L["ldb-menu-item-doflaskcheck-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledBuffMonitorCheckFlasks(nil)))
                     end
      UIDropDownMenu_AddButton(info, level)

      -- Do Battle Elixir Check Toggle.
      HTBrokerSupport:AddMenuToggleButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-dobelixircheck")
      info.checked = HTBrokerSupport:IsEnabled(HTConfig:GetIsEnabledBuffMonitorCheckBattleElixirs(nil))
      info.func    = function()
                       HTConfig:ToggleIsEnabledBuffMonitorCheckBattleElixirs()
                       HTBrokerSupport:DisplayStatus(L["ldb-menu-item-dobelixircheck-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledBuffMonitorCheckBattleElixirs(nil)))
                     end
      UIDropDownMenu_AddButton(info, level)

       -- Do Guardian Elixir Check Toggle.
       HTBrokerSupport:AddMenuToggleButton()
       HTBrokerSupport:AddMenuButtonText("ldb-menu-item-dogelixircheck")
       info.checked = HTBrokerSupport:IsEnabled(HTConfig:GetIsEnabledBuffMonitorCheckGuardianElixirs(nil))
       info.func    = function()
                        HTConfig:ToggleIsEnabledBuffMonitorCheckGuardianElixirs()
                        HTBrokerSupport:DisplayStatus(L["ldb-menu-item-dogelixircheck-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledBuffMonitorCheckGuardianElixirs(nil)))
                      end
      UIDropDownMenu_AddButton(info, level)

       HTBrokerSupport:AddMenuToggleButton()
       HTBrokerSupport:AddMenuButtonText("ldb-menu-item-roguepoisons")
       info.checked = HTBrokerSupport:IsEnabled(HTConfig:GetIsEnabledBuffMonitorCheckPoisons(nil))
       info.func    = function()
                        HTConfig:ToggleIsEnabledBuffMonitorCheckPoisons()
                        HTBrokerSupport:DisplayStatus(L["ldb-menu-item-roguepoisons-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledBuffMonitorCheckPoisons(nil)))
                      end
      UIDropDownMenu_AddButton(info, level)

    end

    -- ========================================================================
    -- == Render the level 2 submenu for: Busy Responder ======================
    -- ========================================================================
    if UIDROPDOWNMENU_MENU_VALUE == "SubmenuBusyResponder" then

      -- General Menu Defaults.
      HTBrokerSupport:AddMenuDefaultConfig()

      -- Create the title of the menu
      HTBrokerSupport:AddMenuTitle(L["ldb-menu-busyresponder-name"])
      UIDropDownMenu_AddButton(info, level)

      -- Empty Menu Item.
      HTBrokerSupport:AddMenuDummyButton()
      UIDropDownMenu_AddButton(info, level)

      -- Show saved messages.
      HTBrokerSupport:AddMenuPushButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-showsavedmsgs")
      info.func = function() HTConsole:BusyResponderShowMessages() end
      UIDropDownMenu_AddButton(info, level)

      -- Clear saved messages.
      HTBrokerSupport:AddMenuPushButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-clearsavedmsgs")
      info.func = function() HTConsole:BusyResponderClearMessages() end
      UIDropDownMenu_AddButton(info, level)

      -- Select Busy Message.
      HTBrokerSupport:AddMenuPushButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-msg-out-busy")
      info.func = function()
                    HTConfig:SetBRMessageOutgoing(L["con-busy-responder-out-msg-busy-01"], L["con-busy-responder-out-msg-busy-01"])
                    HTBrokerSupport:DisplayValue(L["ldb-menu-item-msg-out-busy-txt-name"], HTConsole:GetBusyResponderMessage())
                  end
      UIDropDownMenu_AddButton(info, level)

      -- Select Break Message.
      HTBrokerSupport:AddMenuPushButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-msg-out-break")
      info.func = function()
                    HTConfig:SetBRMessageOutgoing(L["con-busy-responder-out-msg-break-01"], L["con-busy-responder-out-msg-break-01"])
                    HTBrokerSupport:DisplayValue(L["ldb-menu-item-msg-out-break-txt-name"], HTConsole:GetBusyResponderMessage())
                  end
      UIDropDownMenu_AddButton(info, level)

      -- Empty Menu Item.
      HTBrokerSupport:AddMenuDummyButton()
      UIDropDownMenu_AddButton(info, level)

      -- Create the title of the menu
      HTBrokerSupport:AddMenuTitle(L["ldb-menu-quick-config-name"])
      UIDropDownMenu_AddButton(info, level)

      -- Enable Busy Responder.
      HTBrokerSupport:AddMenuToggleButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-brenabled")
      info.checked = HTBrokerSupport:IsEnabled(HTConfig:GetIsEnabledBusyResponder(nil))
      info.func    = function()
                       HTConfig:ToggleIsEnabledBusyResponder()
                       HTBrokerSupport:DisplayStatus(L["ldb-menu-item-brenabled-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledBusyResponder(nil)))
                     end
      UIDropDownMenu_AddButton(info, level)

      -- Is Guild Member Allowed.
      HTBrokerSupport:AddMenuToggleButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-allowguildmember")
      info.checked = HTBrokerSupport:IsEnabled(HTConfig:GetIsGuildMemberAllowed(nil))
      info.func    = function()
                       HTConfig:ToggleIsGuildMemberAllowed()
                       HTBrokerSupport:DisplayStatus(L["ldb-menu-item-allowguildmember-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsGuildMemberAllowed(nil)))
                     end
      UIDropDownMenu_AddButton(info, level)

       -- Is Guild Officer Allowed.
      HTBrokerSupport:AddMenuToggleButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-allowguildofficer")
      info.checked = HTBrokerSupport:IsEnabled(HTConfig:GetIsGuildOfficerAllowed(nil))
      info.func    = function()
                       HTConfig:ToggleIsGuildOfficerAllowed()
                       HTBrokerSupport:DisplayStatus(L["ldb-menu-item-allowguildofficer-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsGuildOfficerAllowed(nil)))
                     end
      UIDropDownMenu_AddButton(info, level)

      -- Is Raid Member Allowed.
      HTBrokerSupport:AddMenuToggleButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-allowraidmember")
      info.checked = HTBrokerSupport:IsEnabled(HTConfig:GetIsRaidMemberAllowed(nil))
      info.func    = function()
                       HTConfig:ToggleIsRaidMemberAllowed()
                       HTBrokerSupport:DisplayStatus(L["ldb-menu-item-allowraidmember-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsRaidMemberAllowed(nil)))
                     end
      UIDropDownMenu_AddButton(info, level)

      -- Is Party Member Allowed.
      HTBrokerSupport:AddMenuToggleButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-allowpartymember")
      info.checked = HTBrokerSupport:IsEnabled(HTConfig:GetIsPartyMemberAllowed(nil))
      info.func    = function()
                       HTConfig:ToggleIsPartyMemberAllowed()
                       HTBrokerSupport:DisplayStatus(L["ldb-menu-item-allowpartymember-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsPartyMemberAllowed(nil)))
                     end
      UIDropDownMenu_AddButton(info, level)

      -- Is Friend Allowed.
      HTBrokerSupport:AddMenuToggleButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-allowfriend")
      info.checked = HTBrokerSupport:IsEnabled(HTConfig:GetIsFriendAllowed(nil))
      info.func    = function()
                       HTConfig:ToggleIsFriendAllowed()
                       HTBrokerSupport:DisplayStatus(L["ldb-menu-item-allowfriend-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsFriendAllowed(nil)))
                     end
      UIDropDownMenu_AddButton(info, level)

      -- Is Real-ID Friend Allowed.
      HTBrokerSupport:AddMenuToggleButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-allowrealidfriend")
      info.checked = HTBrokerSupport:IsEnabled(HTConfig:GetIsRealIDFriendAllowed(nil))
      info.func    = function()
                       HTConfig:ToggleIsRealIDFriendAllowed()
                        HTBrokerSupport:DisplayStatus(L["ldb-menu-item-allowrealidfriend-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsRealIDFriendAllowed(nil)))
                      end
      UIDropDownMenu_AddButton(info, level)
    end

    -- ========================================================================
    -- == Render the level 2 submenu for: Raid Functions ======================
    -- ========================================================================
    if UIDROPDOWNMENU_MENU_VALUE == "SubmenuRaidFunctions" then

      -- General Menu Defaults.
      HTBrokerSupport:AddMenuDefaultConfig()

      -- Create the title of the menu
      HTBrokerSupport:AddMenuTitle(L["ldb-menu-raid-name"])
      UIDropDownMenu_AddButton(info, level)

      -- Empty Menu Item.
      HTBrokerSupport:AddMenuDummyButton()
      UIDropDownMenu_AddButton(info, level)

      -- Begin Invitation Period Item.
      HTBrokerSupport:AddMenuPushButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-begin-invites")
      info.func = function() HalonaTools:BeginRaidInvitationPeriod(UnitName("player")) end
      UIDropDownMenu_AddButton(info, level)

      -- Begin Raid Session Item.
      HTBrokerSupport:AddMenuPushButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-begin-raid")
      info.func = function() HalonaTools:BeginRaidSession(UnitName("player")) end
      UIDropDownMenu_AddButton(info, level)

      -- End Raid Session Item.
      HTBrokerSupport:AddMenuPushButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-end-raid")
      info.func = function() HalonaTools:EndRaidSession(UnitName("player")) end
      UIDropDownMenu_AddButton(info, level)

      -- Empty Menu Item.
      HTBrokerSupport:AddMenuDummyButton()
      UIDropDownMenu_AddButton(info, level)

      -- DBM Timer, Raid Break, 3 mins.
      HTBrokerSupport:AddMenuPushButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-dbm-timer-03m")
      info.func = function()
                    HalonaTools:CreateDBMTimer(180, L["ht-raid-break-timer-01"], true)
                    HTConsole:SendChatMessage(string.gsub(L["ht-raid-break-timer-02"], "TIMER_DURATION", "3"), "RAID", nil, nil)
                  end
      UIDropDownMenu_AddButton(info, level)

      -- DBM Timer, Raid Break, 5 mins.
      HTBrokerSupport:AddMenuPushButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-dbm-timer-05m")
      info.func = function()
                    HalonaTools:CreateDBMTimer(300, L["ht-raid-break-timer-01"], true)
                    HTConsole:SendChatMessage(string.gsub(L["ht-raid-break-timer-02"], "TIMER_DURATION", "5"), "RAID", nil, nil)
                  end
      UIDropDownMenu_AddButton(info, level)

      -- DBM Timer, Raid Break, 10 mins.
      HTBrokerSupport:AddMenuPushButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-dbm-timer-10m")
      info.func = function()
                    HalonaTools:CreateDBMTimer(600, L["ht-raid-break-timer-01"], true)
                    HTConsole:SendChatMessage(string.gsub(L["ht-raid-break-timer-02"], "TIMER_DURATION", "10"), "RAID", nil, nil)
                  end
      UIDropDownMenu_AddButton(info, level)

      -- DBM Timer, Raid Break, 15 mins.
      HTBrokerSupport:AddMenuPushButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-dbm-timer-15m")
      info.func = function()
                    HalonaTools:CreateDBMTimer(900, L["ht-raid-break-timer-01"], true)
                    HTConsole:SendChatMessage(string.gsub(L["ht-raid-break-timer-02"], "TIMER_DURATION", "15"), "RAID", nil, nil)
                  end
      UIDropDownMenu_AddButton(info, level)

      -- Empty Menu Item.
      HTBrokerSupport:AddMenuDummyButton()
      UIDropDownMenu_AddButton(info, level)

      -- DBM Timer, Raid Break, 20 mins.
      HTBrokerSupport:AddMenuPushButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-dbm-timer-20m")
      info.func = function()
                    HalonaTools:CreateDBMTimer(1200, L["ht-raid-break-timer-01"], true)
                    HTConsole:SendChatMessage(string.gsub(L["ht-raid-break-timer-02"], "TIMER_DURATION", "20"), "RAID", nil, nil)
                  end
      UIDropDownMenu_AddButton(info, level)

      -- DBM Timer, Raid Break, 25 mins.
      HTBrokerSupport:AddMenuPushButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-dbm-timer-25m")
      info.func = function()
                    HalonaTools:CreateDBMTimer(1500, L["ht-raid-break-timer-01"], true)
                    HTConsole:SendChatMessage(string.gsub(L["ht-raid-break-timer-02"], "TIMER_DURATION", "25"), "RAID", nil, nil)
                  end
      UIDropDownMenu_AddButton(info, level)

      -- DBM Timer, Raid Break, 30 mins.
      HTBrokerSupport:AddMenuPushButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-dbm-timer-30m")
      info.func = function()
                    HalonaTools:CreateDBMTimer(1800, L["ht-raid-break-timer-01"], true)
                    HTConsole:SendChatMessage(string.gsub(L["ht-raid-break-timer-02"], "TIMER_DURATION", "30"), "RAID", nil, nil)
                  end
      UIDropDownMenu_AddButton(info, level)

    end

    -- ========================================================================
    -- == Render the level 2 submenu for: Invitations =========================
    -- ========================================================================
    if UIDROPDOWNMENU_MENU_VALUE == "SubmenuInvitations" then

      -- General Menu Defaults.
      HTBrokerSupport:AddMenuDefaultConfig()

      -- Create the title of the menu
      HTBrokerSupport:AddMenuTitle(L["ldb-menu-invitations-name"])
      UIDropDownMenu_AddButton(info, level)

      -- Empty Menu Item.
      HTBrokerSupport:AddMenuDummyButton()
      UIDropDownMenu_AddButton(info, level)

      -- Create the title of the menu
      HTBrokerSupport:AddMenuTitle(L["ldb-menu-quick-config-name"])
      UIDropDownMenu_AddButton(info, level)

      -- Auto Invitations Toggle.
      HTBrokerSupport:AddMenuToggleButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-autoinvitations")
      info.checked = HTBrokerSupport:IsEnabled(HTConfig:GetIsEnabledAutoInvitations(nil))
      info.func    = function()
                       HTConfig:ToggleIsEnabledAutoInvitations()
                       HTBrokerSupport:DisplayStatus(L["ldb-menu-item-autoinvitations-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledAutoInvitations(nil)))
                     end
      UIDropDownMenu_AddButton(info, level)

      -- Auto Raid Promotions Toggle.
      HTBrokerSupport:AddMenuToggleButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-autoraidpromotions")
      info.checked = HTBrokerSupport:IsEnabled(HTConfig:GetIsEnabledAutoRaidPromotions(nil))
      info.func    = function()
                       HTConfig:ToggleIsEnabledAutoRaidPromotions()
                       HTBrokerSupport:DisplayStatus(L["ldb-menu-item-autoraidpromotions-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledAutoRaidPromotions(nil)))
                     end
      UIDropDownMenu_AddButton(info, level)

      -- Auto Raid Invitation Announcement Response.
      HTBrokerSupport:AddMenuToggleButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-autoriaresponse")
      info.checked = HTBrokerSupport:IsEnabled(HTConfig:GetIsEnabledAutoRIAResponse(nil))
      info.func    = function()
                       HTConfig:ToggleIsEnabledAutoRIAResponse()
                       HTBrokerSupport:DisplayStatus(L["ldb-menu-item-autoriaresponse-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledAutoRIAResponse(nil)))
                     end
      UIDropDownMenu_AddButton(info, level)

      -- Auto Raid Invitation Announcement Response.
      HTBrokerSupport:AddMenuToggleButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-autorqaresponse")
      info.checked = HTBrokerSupport:IsEnabled(HTConfig:GetIsEnabledAutoRQAResponse(nil))
      info.func    = function()
                       HTConfig:ToggleIsEnabledAutoRQAResponse()
                       HTBrokerSupport:DisplayStatus(L["ldb-menu-item-autorqaresponse-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledAutoRQAResponse(nil)))
                     end
      UIDropDownMenu_AddButton(info, level)
    end

    -- ========================================================================
    -- == Render the level 2 submenu for: Recruitment =========================
    -- ========================================================================
    if UIDROPDOWNMENU_MENU_VALUE == "SubmenuRecruitment" then

      -- General Menu Defaults.
      HTBrokerSupport:AddMenuDefaultConfig()

      -- Create the title of the menu
      HTBrokerSupport:AddMenuTitle(L["ldb-menu-recruitment-name"])
      UIDropDownMenu_AddButton(info, level)

      -- Empty Menu Item.
      HTBrokerSupport:AddMenuDummyButton()
      UIDropDownMenu_AddButton(info, level)

      -- Set RMS Interval to 05 mins.
      HTBrokerSupport:AddMenuPushButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-rms-interval-05m")
      info.func = function()
                    HTConfig:SetRecruitmentInterval(360, 360)
                    HTBrokerSupport:DisplayValue(L["ldb-menu-item-rms-interval-05m-txt-name"], 360)
                  end
      UIDropDownMenu_AddButton(info, level)

      -- Set RMS Interval to 10 mins.
      HTBrokerSupport:AddMenuPushButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-rms-interval-10m")
      info.func = function()
                    HTConfig:SetRecruitmentInterval(720, 720)
                    HTBrokerSupport:DisplayValue(L["ldb-menu-item-rms-interval-10m-txt-name"], 720)
                  end
      UIDropDownMenu_AddButton(info, level)

      -- Set RMS Interval to 15 mins.
      HTBrokerSupport:AddMenuPushButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-rms-interval-15m")
      info.func = function()
                    HTConfig:SetRecruitmentInterval(900, 900)
                    HTBrokerSupport:DisplayValue(L["ldb-menu-item-rms-interval-15m-txt-name"], 900)
                  end
      UIDropDownMenu_AddButton(info, level)

      -- Empty Menu Item.
      HTBrokerSupport:AddMenuDummyButton()
      UIDropDownMenu_AddButton(info, level)

      -- Send RMS Now Button.
      HTBrokerSupport:AddMenuPushButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-manualrms")
      info.func = function()
                    HTConsole:BroadcastMessageRMS()
                  end
      UIDropDownMenu_AddButton(info, level)

      -- Empty Menu Item.
      HTBrokerSupport:AddMenuDummyButton()
      UIDropDownMenu_AddButton(info, level)

      -- Create the title of the menu
      HTBrokerSupport:AddMenuTitle(L["ldb-menu-quick-config-name"])
      UIDropDownMenu_AddButton(info, level)

      -- Recruitment Message Sender Toggle.
      HTBrokerSupport:AddMenuToggleButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-autorms")
      info.checked = HTBrokerSupport:IsEnabled(HTConfig:GetIsEnabledRecruitmentMessageSender(nil))
      info.func    = function()
                       HTConfig:ToggleIsEnabledRecruitmentMessageSender()
                       HTBrokerSupport:DisplayStatus(L["ldb-menu-item-autorms-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledRecruitmentMessageSender(nil)))
                       HTConsole:ToggleRMSTimerHelper()
                     end
      UIDropDownMenu_AddButton(info, level)

      -- RMS Group Lock.
      HTBrokerSupport:AddMenuToggleButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-rms-grouplock")
      info.checked = HTBrokerSupport:IsEnabled(HTConfig:GetIsEnabledRMSGroupLock(nil))
      info.func    = function()
                       HTConfig:ToggleIsEnabledRMSGroupLock()
                       HTBrokerSupport:DisplayStatus(L["ldb-menu-item-rms-grouplock-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledRMSGroupLock(nil)))
                     end
      UIDropDownMenu_AddButton(info, level)

      -- RMS Guild Lock.
      HTBrokerSupport:AddMenuToggleButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-rms-guildlock")
      info.checked = HTBrokerSupport:IsEnabled(HTConfig:GetIsEnabledRMSGuildLocked(nil))
      info.func    = function()
                       HTConfig:ToggleIsEnabledRMSGuildLocked()
                       HTBrokerSupport:DisplayStatus(L["ldb-menu-item-rms-guildlock-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledRMSGuildLocked(nil)))
                     end
      UIDropDownMenu_AddButton(info, level)

      -- Empty Menu Item.
      HTBrokerSupport:AddMenuDummyButton()
      UIDropDownMenu_AddButton(info, level)

      -- Show Preferred Guild Button.
      HTBrokerSupport:AddMenuPushButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-rms-showguild")
      info.func = function()
                    HTBrokerSupport:DisplayValue(L["ldb-menu-item-rms-showguild-txt-name"], HTConfig:GetPreferredRMSGuild(nil))
                  end
      UIDropDownMenu_AddButton(info, level)

      -- Set Preferred Guild Button.
      HTBrokerSupport:AddMenuPushButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-rms-setguild")
      info.func = function()
                    HTConsole:UpdatePreferredRMSGuild()
                    HTBrokerSupport:DisplayValue(L["ldb-menu-item-rms-setguild-txt-name"], HTConfig:GetPreferredRMSGuild(nil))
                  end
      UIDropDownMenu_AddButton(info, level)
    end

    -- ========================================================================
    -- == Render the level 2 submenu for: System ==============================
    -- ========================================================================
    if UIDROPDOWNMENU_MENU_VALUE == "SubmenuSystem" then

      -- General Menu Defaults.
      HTBrokerSupport:AddMenuDefaultConfig()

      -- Create the title of the menu
      HTBrokerSupport:AddMenuTitle(L["ldb-menu-system-name"])
      UIDropDownMenu_AddButton(info, level)

      -- Empty Menu Item.
      HTBrokerSupport:AddMenuDummyButton()
      UIDropDownMenu_AddButton(info, level)

      -- Show Report Addon Usage (CHAT).
      HTBrokerSupport:AddMenuPushButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-addon-usage-chat")
      info.func = function()
                    HalonaTools:ShowReportUsageStatus("CHAT")
                  end
      UIDropDownMenu_AddButton(info, level)

      -- Show Report Addon Usage (GUILD).
      HTBrokerSupport:AddMenuPushButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-addon-usage-guild")
      info.func = function()
                    HalonaTools:ShowReportUsageStatus("GUILD")
                  end
      UIDropDownMenu_AddButton(info, level)

      -- Show Report Addon Usage (RAID).
      HTBrokerSupport:AddMenuPushButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-addon-usage-raid")
      info.func = function()
                    HalonaTools:ShowReportUsageStatus("RAID")
                  end
      UIDropDownMenu_AddButton(info, level)

      -- Empty Menu Item.
      HTBrokerSupport:AddMenuDummyButton()
      UIDropDownMenu_AddButton(info, level)

      -- Addon Version Broadcast.
      HTBrokerSupport:AddMenuPushButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-avb")
      info.func = function()
                    HTConsole:BroadcastMessageAVB()
                  end
      UIDropDownMenu_AddButton(info, level)

      -- Addon Version Query.
      HTBrokerSupport:AddMenuPushButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-avq")
      info.func = function()
                    HTConsole:BroadcastMessageAVQ(L["ldb-menu-item-avq-messagetext"])
                  end
      UIDropDownMenu_AddButton(info, level)

      -- Empty Menu Item.
      HTBrokerSupport:AddMenuDummyButton()
      UIDropDownMenu_AddButton(info, level)

      -- Set Heartbeat Interval to Once Every 05m.
      HTBrokerSupport:AddMenuPushButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-heartbeat-05m")
      info.func = function()
                    HTConfig:SetHeartbeatInterval(300, 300)
                    HTBrokerSupport:DisplayValue(L["ldb-menu-item-heartbeat-05m-txt-name"], 300)
                  end
      UIDropDownMenu_AddButton(info, level)

      -- Set Heartbeat Interval to Once Every 10m.
      HTBrokerSupport:AddMenuPushButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-heartbeat-10m")
      info.func = function()
                    HTConfig:SetHeartbeatInterval(600, 600)
                    HTBrokerSupport:DisplayValue(L["ldb-menu-item-heartbeat-10m-txt-name"], 600)
                  end
      UIDropDownMenu_AddButton(info, level)

      -- Set Heartbeat Interval to Once Every 15m.
      HTBrokerSupport:AddMenuPushButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-heartbeat-15m")
      info.func = function()
                    HTConfig:SetHeartbeatInterval(900, 900)
                    HTBrokerSupport:DisplayValue(L["ldb-menu-item-heartbeat-15m-txt-name"], 900)
                  end
      UIDropDownMenu_AddButton(info, level)

      -- Empty Menu Item.
      HTBrokerSupport:AddMenuDummyButton()
      UIDropDownMenu_AddButton(info, level)

      -- Create the title of the menu
      HTBrokerSupport:AddMenuTitle(L["ldb-menu-quick-config-name"])
      UIDropDownMenu_AddButton(info, level)

      -- Alert New Addon Version Toggle.
      HTBrokerSupport:AddMenuToggleButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-alertnewaddonversion")
      info.checked = HTBrokerSupport:IsEnabled(HTConfig:GetIsEnabledAlertNewAddonVersion(nil))
      info.func    = function()
                       HTConfig:ToggleIsEnabledAlertNewAddonVersion()
                       HTBrokerSupport:DisplayStatus(L["ldb-menu-item-alertnewaddonversion-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledAlertNewAddonVersion(nil)))
                     end
      UIDropDownMenu_AddButton(info, level)

      -- Debug Mode Toggle.
      HTBrokerSupport:AddMenuToggleButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-debugmode")
      info.checked = HTBrokerSupport:IsEnabled(HTConfig:GetIsEnabledDebugMode(nil))
      info.func    = function()
                       HTConfig:ToggleIsEnabledDebugMode()
                       HTBrokerSupport:DisplayStatus(L["ldb-menu-item-debugmode-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledDebugMode(nil)))
                     end
      UIDropDownMenu_AddButton(info, level)

      -- Debug Filter Toggle.
      HTBrokerSupport:AddMenuToggleButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-debugfilter")
      info.checked = HTBrokerSupport:IsEnabled(HTConfig:GetIsEnabledDebugFilter(nil))
      info.func    = function()
                       HTConfig:ToggleIsEnabledDebugFilter()
                       HTBrokerSupport:DisplayStatus(L["ldb-menu-item-debugfilter-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledDebugFilter(nil)))
                     end
      UIDropDownMenu_AddButton(info, level)

      -- General Menu Defaults.
      HTBrokerSupport:AddMenuDefaultConfig()

      -- Debug Filters Submenu.
      HTBrokerSupport:AddMenuSubMenu("SubmenuDebugFilters")
      HTBrokerSupport:AddMenuButtonText("ldb-menu-heading-debugfilters")
      UIDropDownMenu_AddButton(info, level)

      -- Empty Menu Item.
      HTBrokerSupport:AddMenuDummyButton()
      UIDropDownMenu_AddButton(info, level)

      -- Config Menu Item.
      HTBrokerSupport:AddMenuPushButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-config")
      info.func = function()
                    HTGUI:Config()
                  end
      UIDropDownMenu_AddButton(info, level)
    end

  -- ==========================================================================
  -- == Level 3 - Submenus linked from 2nd level menus.                      ==
  -- ==========================================================================
  elseif level == 3 then

    -- ========================================================================
    -- == Render the level 3 submenu for: Debug Filters =======================
    -- ========================================================================
    if UIDROPDOWNMENU_MENU_VALUE == "SubmenuDebugFilters" then

      -- General Menu Defaults.
      HTBrokerSupport:AddMenuDefaultConfig()

      -- Create the title of the menu
      HTBrokerSupport:AddMenuTitle(L["ldb-menu-debugfilters-name"])
      UIDropDownMenu_AddButton(info, level)

      -- Empty Menu Item.
      HTBrokerSupport:AddMenuDummyButton()
      UIDropDownMenu_AddButton(info, level)

      -- Set Debug Filter for Buff Announcer output.
      HTBrokerSupport:AddMenuPushButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-debugfilter-ba")
      info.func = function()
                    HTConfig:SetDebugFilter(L["ldb-menu-item-debugfilter-ba-data-val"], L["ldb-menu-item-debugfilter-ba-data-val"])
                    HTBrokerSupport:DisplayValue(L["ldb-menu-item-debugfilter-ba-txt-name"], L["ldb-menu-item-debugfilter-ba-data-val"])
                  end
      UIDropDownMenu_AddButton(info, level)

      -- Set Debug Filter for Buff Checker output.
      HTBrokerSupport:AddMenuPushButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-debugfilter-bc")
      info.func = function()
                    HTConfig:SetDebugFilter(L["ldb-menu-item-debugfilter-bc-data-val"], L["ldb-menu-item-debugfilter-bc-data-val"])
                    HTBrokerSupport:DisplayValue(L["ldb-menu-item-debugfilter-bc-txt-name"], L["ldb-menu-item-debugfilter-bc-data-val"])
                  end
      UIDropDownMenu_AddButton(info, level)

      -- Set Debug Filter for Busy Responder output.
      HTBrokerSupport:AddMenuPushButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-debugfilter-br")
      info.func = function()
                    HTConfig:SetDebugFilter(L["ldb-menu-item-debugfilter-br-data-val"], L["ldb-menu-item-debugfilter-br-data-val"])
                    HTBrokerSupport:DisplayValue(L["ldb-menu-item-debugfilter-br-txt-name"], L["ldb-menu-item-debugfilter-br-data-val"])
                  end
      UIDropDownMenu_AddButton(info, level)

      -- Set Debug Filter for Recruitment output.
      HTBrokerSupport:AddMenuPushButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-debugfilter-rms")
      info.func = function()
                    HTConfig:SetDebugFilter(L["ldb-menu-item-debugfilter-rms-data-val"], L["ldb-menu-item-debugfilter-rms-data-val"])
                    HTBrokerSupport:DisplayValue(L["ldb-menu-item-debugfilter-rms-txt-name"], L["ldb-menu-item-debugfilter-rms-data-val"])
                  end
      UIDropDownMenu_AddButton(info, level)

    end

  end -- The indentation messes up here because this entire block is a
  end -- function definition being fed into the DropDownMenu.initialize declaration.
end

-- ============================================================================
-- == AddMenuDefaultConfig()                                                 ==
-- ==                                                                        ==
-- == This function adds the default meta-data required by all menu items.   ==
-- ============================================================================

function HTBrokerSupport:AddMenuDefaultConfig()

  info.keepShownOnClick = 1

end

-- ============================================================================
-- == AddMenuTitle(title)                                                    ==
-- ==                                                                        ==
-- == This function adds the meta-data required for a new menu title.        ==
-- ============================================================================

function HTBrokerSupport:AddMenuTitle(title)

  -- Create the title of the menu
  info.isTitle      = 1
  info.text         = title
  info.notCheckable = 1

end

-- ============================================================================
-- == AddMenuDummyButton()                                                   ==
-- ==                                                                        ==
-- == This function adds the meta-data required for a new dummy button.      ==
-- ============================================================================

function HTBrokerSupport:AddMenuDummyButton()

  -- Empty Menu Item.
  info.text         = " "
  info.checked      = nil
  info.notCheckable = 1
  info.hasArrow     = nil
  info.value        = nil
  info.disabled     = 1

end

-- ============================================================================
-- == AddMenuSubMenu()                                                       ==
-- ==                                                                        ==
-- == This function adds the meta-data required for a new sub-menu.          ==
-- ============================================================================

function HTBrokerSupport:AddMenuSubMenu(name)

  info.isTitle      = nil
  info.disabled     = nil
  info.func         = self.UncheckHack
  info.hasArrow     = 1
  info.value        = name
  info.notCheckable = 1

end

-- ============================================================================
-- == AddMenuPushButton()                                                    ==
-- ==                                                                        ==
-- == This function adds the meta-data required for a new push button.       ==
-- ============================================================================

function HTBrokerSupport:AddMenuPushButton()

  info.isTitle      = nil
  info.disabled     = nil
  info.hasArrow     = nil
  info.value        = nil
  info.notCheckable = 1

end

-- ============================================================================
-- == AddMenuToggleButton()                                                  ==
-- ==                                                                        ==
-- == This function adds the meta-data required for a new toggle button.     ==
-- ============================================================================

function HTBrokerSupport:AddMenuToggleButton()

  info.isTitle      = nil
  info.disabled     = nil
  info.hasArrow     = nil
  info.notCheckable = nil

end

-- ============================================================================
-- == AddMenuButtonText(referenceStub)                                       ==
-- ==                                                                        ==
-- == This function adds menu button text fields to the 'info' table for the ==
-- == supplied referenceStub value.                                          ==
-- ============================================================================

function HTBrokerSupport:AddMenuButtonText(localeString)

  local txtName = localeString .. "-txt-name"
  local tipName = localeString .. "-tip-name"
  local tipDesc = localeString .. "-tip-desc"

  info.text         = L[txtName]
  info.tooltipTitle = L[tipName]
  info.tooltipText  = L[tipDesc]

end

-- ============================================================================
-- == IsEnabled(value)                                                       ==
-- ==                                                                        ==
-- == This function determines whether the supplied value is recognised as   ==
-- == meaning "is enabled" and returns '1' if enabled, nil otherwise.        ==
-- ============================================================================

function HTBrokerSupport:IsEnabled(value)

  if value == true or value == 1 then
    return 1
  else
    return nil
  end

end

-- ============================================================================
-- == IsEnabledText(value)                                                   ==
-- ==                                                                        ==
-- == This function determines whether the supplied value is recognised as   ==
-- == meaning "is enabled" and returns 'On' if enabled, 'Off' otherwise.     ==
-- ============================================================================

function HTBrokerSupport:IsEnabledText(value)

  if HTBrokerSupport:IsEnabled(value) then
    return "|cff00ff00On|r"
  else
    return "|cffff0000Off|r"
  end

end

-- ============================================================================
-- == DisplayStatus(featureName, isEnabledText)                              ==
-- ==                                                                        ==
-- == This function enables the status of a feature to be displayed via the  ==
-- == standard chat-window system with appropriate colourisation.            ==
-- ============================================================================

function HTBrokerSupport:DisplayStatus(featureName, isEnabledText)

  local message = L["cfg-display-data-title"] .. featureName .. ": " .. isEnabledText

  HTConsole:SendChatMessage(message, "CHAT", nil, nil)

end

-- ============================================================================
-- == DisplayValue(featureName, value)                                       ==
-- ==                                                                        ==
-- == This function enables the value of a feature to be displayed via the   ==
-- == standard chat-window system.                                           ==
-- ============================================================================

function HTBrokerSupport:DisplayValue(featureName, value)

  local message = L["cfg-display-data-title"] .. featureName .. ": |cffffff00" .. value .. "|r"

  HTConsole:SendChatMessage(message, "CHAT", nil, nil)

end

-- ============================================================================
-- == GetTooltipOfficerList()                                                ==
-- ==                                                                        ==
-- == This function generates the officer list required for display in the   ==
-- == LDB tooltip.                                                           ==
-- ============================================================================

function HTBrokerSupport:GetTooltipOfficerList()

  local rawList            = HTCommon:GetGuildOfficerList()
  local onlineOfficers     = {}
  local offlineOfficers    = {}
  local tooltipOfficerList = {}

  for i,v in ipairs(rawList) do

    local characterName = HTCommon:FindPlayerByCharacterName(v)
    local testName      = characterName -- Needed because string.find alters string.

    if string.match(testName, L["ldb-tooltips-officers-offline-query"]) then
      table.insert(offlineOfficers, "|cff990000" .. characterName .. "|r")
    else
      table.insert(onlineOfficers, "|cff00ff00" .. characterName .. "|r")
    end

    table.sort(onlineOfficers)
    table.sort(offlineOfficers)

  end

  tooltipOfficerList = HTData:JoinTables(onlineOfficers, offlineOfficers)

  return tooltipOfficerList

end
