-- ============================================================================
-- == HTGUI - Provides support to HalonaTools for managing GUI resources.    ==
-- ============================================================================

HTGUI        = LibStub("AceAddon-3.0"):NewAddon("HTGUI", "AceConsole-3.0", "AceEvent-3.0")
local L      = LibStub("AceLocale-3.0"):GetLocale("HalonaTools")

local AceGUI = LibStub("AceGUI-3.0")

-- ============================================================================
-- == OnInitialize()                                                         ==
-- ==                                                                        ==
-- == This function is called when the addon is initialised.                 ==
-- ============================================================================

function HTGUI:OnInitialize()
end

-- ============================================================================
-- == OnEnable()                                                             ==
-- ==                                                                        ==
-- == This function is called when the addon is enabled.                     ==
-- ============================================================================

function HTGUI:OnEnable()
end

-- ============================================================================
-- == OnEnable()                                                             ==
-- ==                                                                        ==
-- == This function is called when the addon is disabled.                    ==
-- ============================================================================

function HTGUI:OnDisable()
end

-- ============================================================================
-- == AboutHalonaTools()                                                     ==
-- ==                                                                        ==
-- == This function displays a standard "About" window, providing            ==
-- == information about the HalonaTools addon.                               ==
-- ============================================================================

function HTGUI:About()

  local frame = AceGUI:Create("Frame")
  frame:SetTitle("About")
  frame:SetStatusText("(c) aLL I EVER WANTED productions, 2011.")
  frame:SetCallback("OnClose", function(widget) AceGUI:Release(widget) end)

end

-- ============================================================================
-- == ShowChange(fieldName, value)                                           ==
-- ==                                                                        ==
-- == This function displays a message indicating that a configuration value ==
-- == has changed, indicating which field was changed and what the new value ==
-- == of the field has become.                                               ==
-- ============================================================================

function HTGUI:ShowChange(fieldName, value)

  HTConsole:SendChatMessage("[HT] - " .. fieldName .. " - " .. value, "CHAT", nil, nil)

end

-- ============================================================================
-- == BuildLengthAdvisoryLabel(name, value, maxLength)                       ==
-- ==                                                                        ==
-- == This function builds a text string that may be used as a field label   ==
-- == that incorporates advice on the current field length and the maximum   ==
-- == permitted field length.                                                ==
-- ==                                                                        ==
-- == Example: "RMS Message (32/255)"                                        ==
-- ============================================================================

function HTGUI:BuildLengthAdvisoryLabel(name, value, maxLength)

  local valueLength = string.len(tostring(value))
  local labelText = name .. " (" .. tostring(valueLength) .. "/" .. tostring(maxLength) .. ")"

  return labelText

end

-- ============================================================================
-- == Config()                                                               ==
-- ==                                                                        ==
-- == This function displays the configuration window for the HalonaTools    ==
-- == addon.                                                                 ==
-- ============================================================================

function HTGUI:Config()

  -- function that draws the widgets for the first tab
  local function DrawGroup1(container)
    local desc = AceGUI:Create("Label")
    desc:SetText("Recruitment Message Sender")
    desc:SetFullWidth(true)
    container:AddChild(desc)

    local button = AceGUI:Create("Button")
    button:SetText("Show Current RMS Message")
    button:SetWidth(200)
    button:SetCallback("OnClick", function() HTGUI:ShowChange("RMS Message", HTConfig:GetRecruitmentMessage(nil)) end)
    container:AddChild(button)

    local editbox = AceGUI:Create("EditBox")
    editbox:SetLabel(HTGUI:BuildLengthAdvisoryLabel("Recruitment Message", HTConfig:GetRecruitmentMessage(nil), 255))
    editbox:SetWidth(200)
    editbox:SetText(HTConfig:GetRecruitmentMessage(nil))
    editbox:SetMaxLetters(255)
    editbox:SetCallback("OnEnterPressed", function(widget, event, text) HTConfig:SetRecruitmentMessage(text, text) end)
    -- editbox:SetCallback("OnTextChanged", function(widget, event, text) editbox:SetLabel(HTGUI:BuildLengthAdvisoryLabel("Recruitment Message", HTConfig:GetRecruitmentMessage(nil), 255)) end)
    container:AddChild(editbox)
  end

  -- function that draws the widgets for the second tab
  local function DrawGroup2(container)
    local desc = AceGUI:Create("Label")
    desc:SetText("Console")
    desc:SetFullWidth(true)
    container:AddChild(desc)

    local conTextMaxWidthSlider = AceGUI:Create("Slider")
    conTextMaxWidthSlider:SetValue(HTConfig:GetChatWindowTextMaxWidth(nil))
    conTextMaxWidthSlider:SetSliderValues(30, 100, 5)
    conTextMaxWidthSlider:SetLabel("Max Chat-Window Text Width")
    conTextMaxWidthSlider:SetCallback("OnMouseUp", function(widget, event, text) HTConsole:Debug("Slider Value..." .. text) HTConfig:SetChatWindowTextMaxWidth(tonumber(text), tonumber(text)) end)
    container:AddChild(conTextMaxWidthSlider)

    local button = AceGUI:Create("Button")
    button:SetText("Test Setting")
    button:SetWidth(200)
    button:SetCallback("OnClick", function() HTConsole:SendTitleBar("CHAT") end)
    container:AddChild(button)
  end

  -- Callback function for OnGroupSelected
  local function SelectGroup(container, event, group)
     container:ReleaseChildren()
     if group == "tab1" then
        DrawGroup1(container)
     elseif group == "tab2" then
        DrawGroup2(container)
     end
  end

  -- Create the frame container
  local frame = AceGUI:Create("Frame")
  frame:SetTitle("HT-Config")
  frame:SetStatusText("HalonaTools Configuration GUI")
  frame:SetCallback("OnClose", function(widget) AceGUI:Release(widget) end)
  -- Fill Layout - the TabGroup widget will fill the whole frame
  frame:SetLayout("Fill")

  -- Create the TabGroup
  local tab =  AceGUI:Create("TabGroup")
  tab:SetLayout("Flow")
  -- Setup which tabs to show
  tab:SetTabs({{text="Recruitment Message Sender", value="tab1"}, {text="Console Configuration", value="tab2"}})
  -- Register callback
  tab:SetCallback("OnGroupSelected", SelectGroup)
  -- Set initial Tab (this will fire the OnGroupSelected callback)
  tab:SelectTab("tab1")

  -- add to the frame container
  frame:AddChild(tab)

end


-- ============================================================================
-- == PopupAskRequestRaidInvitation(raidInviter, messageAnnouncer)           ==
-- ==                                                                        ==
-- == This function displays a static popup dialog asking the player if they ==
-- == wish to request a raid invitation from the current raid inviter and    ==
-- == processes the response they make.                                      ==
-- ============================================================================

function HTGUI:PopupAskRequestRaidInvitation(raidInviter, messageAnnouncer)

  -- == ----------------------------------------------------------------------
  -- == -- Configure the text strings...                                    --
  -- == ----------------------------------------------------------------------
  local windowTitle   = string.gsub(L["gui-ria-window-msg-01"], "RIA_INVITER", raidInviter)
  local joinMessage   = L["gui-ria-msg-first"] .. L["gui-ria-join-msg-01"]
  local cancelMessage = L["gui-ria-msg-first"] .. L["gui-ria-cancel-msg-01"]

  cancelMessage = string.gsub(cancelMessage, "RIA_KEYWORD", L["gui-ria-keyword"])
  cancelMessage = string.gsub(cancelMessage, "RIA_INVITER", raidInviter)

  -- == ----------------------------------------------------------------------
  -- == -- Configure the popup window...                                    --
  -- == ----------------------------------------------------------------------
  StaticPopupDialogs[L["gui-ria-window-title"]] = {

    text         = windowTitle,
    button1      = L["gui-ria-window-btn-yes"],
    button2      = L["gui-ria-window-btn-no"],
    timeout      = 30,
    whileDead    = 1,
    hideOnEscape = 0,

    OnAccept = function()
      HTConsole:SendChatMessage(L["gui-ria-keyword"], "WHISPER", nil, raidInviter)
      HTConsole:SendChatMessage(joinMessage, "CHAT", nil, nil)
    end,

    OnCancel = function()
      HTConsole:SendChatMessage(cancelMessage, "CHAT", nil, nil)
    end
  }

  -- == ----------------------------------------------------------------------
  -- == -- Display the popup window...                                      --
  -- == ----------------------------------------------------------------------
  if HTConfig:GetIsEnabledAutoRIAResponse(nil) then
    StaticPopup_Show (L["gui-ria-window-title"])
    PlaySound("DwarfExploration")
  end

end

-- ============================================================================
-- == PopupAskRequestRaidQueueListing(raidInviter, messageAnnouncer)         ==
-- ==                                                                        ==
-- == This function displays a static popup dialog asking the player if they ==
-- == wish to request a raid queue listing from the current raid queue       ==
-- == manager and processes the response they make.                          ==
-- ============================================================================

function HTGUI:PopupAskRequestRaidQueueListing(raidInviter, messageAnnouncer)

  -- == ----------------------------------------------------------------------
  -- == -- Configure the text strings...                                    --
  -- == ----------------------------------------------------------------------
  local windowTitle   = string.gsub(L["gui-rqa-window-msg-01"], "RQA_INVITER", raidInviter)
  local joinMessage   = L["gui-rqa-msg-first"] .. L["gui-rqa-join-msg-01"]
  local cancelMessage = L["gui-rqa-msg-first"] .. L["gui-rqa-cancel-msg-01"]

  cancelMessage = string.gsub(cancelMessage, "RQA_KEYWORD", L["gui-rqa-keyword"])
  cancelMessage = string.gsub(cancelMessage, "RQA_INVITER", raidInviter)

  -- == ----------------------------------------------------------------------
  -- == -- Configure the popup window...                                    --
  -- == ----------------------------------------------------------------------
  StaticPopupDialogs[L["gui-rqa-window-title"]] = {

    text         = windowTitle,
    button1      = L["gui-rqa-window-btn-yes"],
    button2      = L["gui-rqa-window-btn-no"],
    timeout      = 30,
    whileDead    = 1,
    hideOnEscape = 0,

    OnAccept = function()
      HTConsole:SendChatMessage(L["gui-rqa-keyword"], "WHISPER", nil, raidInviter)
      HTConsole:SendChatMessage(joinMessage, "CHAT", nil, nil)
    end,

    OnCancel = function()
      HTConsole:SendChatMessage(cancelMessage, "CHAT", nil, nil)
    end
  }

  -- == ----------------------------------------------------------------------
  -- == -- Display the popup window...                                      --
  -- == ----------------------------------------------------------------------
  if HTConfig:GetIsEnabledAutoRIAResponse(nil) then
    StaticPopup_Show (L["gui-rqa-window-title"])
    PlaySound("DwarfExploration")
  end

end
